/**
  SwitchHosts
  Copyright (C) 2009-2009 Takashi Ishikawa.
  http://switchhosts.mozdev.org/

  This source code is released under the GPL license,
  available in the LICENSE file at the root of this installation
  and also online at http://www.gnu.org/licenses/gpl.txt
**/

Components.utils.import("resource://modules/PaidonUtil.jsm");
Components.utils.import("resource://modules/PaidonLogger.jsm");
Components.utils.import("resource://modules/SWHostsFile.jsm");
Components.utils.import("resource://modules/SWFactory.jsm");

function HostsFileEditDialog(){

  this.init = function(){

    this.sOperation = window.arguments[0];
    this.sHostsURI = window.arguments[1];

    this.logger = SWFactory.getLogger(this);

    //ʗvf
    this.hostsTree = document.getElementById('hostsTree');
    this.nameTextBox = document.getElementById('hostsName');

    //CX^Xϐ
    this.records = new Array();

    //ONX
    if(this.sOperation == "edit" || this.sOperation == "copy"){
      var oHosts = SWG.readHosts(this.sHostsURI);
      //deep copyicopy̏ꍇɁACurrentHosts؂ւĂ܂߁j
      this.hostsFile = oHosts.clone();
    }else{
      this.hostsFile = new SWHostsFile();
    }

    this.nameTextBox.value = this.hostsFile.getName();
    this.records = this.hostsFile.getRecords();
    this.hostsTreeView = new HostsFileTreeView(this.records, this);
    this.hostsTree.view = this.hostsTreeView;

    sizeToContent();
  };

  this.create = function(isNew){
    var emptyRecord = new SWHostsFileRecord(" ");
    this.hostsTreeView.appendItem(emptyRecord);
  };

  this.remove = function(){
    var indexes = this.hostsTreeView.selectedIndexes();
    for(var i=0; i<indexes.length; i++){
      this.hostsTreeView.removeItemAt(indexes[i]);
    }
  };

  this.save = function(){

    var name = this.nameTextBox.value;
    if(!this.validateName(name)){
      return false;
    }

    if(this.hostsTreeView.rowCount == 0){
      SWG.alertErrorMessage("error.record.zero");
      return false;
    }

    for(i=0; i<this.hostsTreeView.records.length; i++){
      var record = this.hostsTreeView.records[i];
      if(!this.validateIp(record.getIp())) return false;
      if(!this.validateHost(record.getHost())) return false;
      if(!this.validateComment(record.getComment())) return false;
    }

    if(this.sOperation == "add" || this.sOperation == "copy"){
      //dG[
      var hostsFileList = SWG.getHostsList();

      for(var i = 0; i < hostsFileList.length; i++){
        if(hostsFileList[i].getName() == name){
          SWG.alertErrorMessage("error.name.duplicate");
          return false;
        }
      }
    }

    this.hostsFile.setName(name);

    //ۑ
    SWG.saveHosts(this.hostsFile, this.sOperation);
    SWG.alertInfoMessage("hosts.save.success");

    return true;

  };

  this.validateName = function(text) {
    if(PaidonUtil.trim(text) == ""){
      SWG.alertErrorMessage("error.name.empty");
      return false;
    }
    return true;
  };

  this.validateIp = function(text) {
    if(PaidonUtil.trim(text) == ""){
      SWG.alertErrorMessage("error.ip.empty");
      return false;
    }
    if(text.match(/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/g) != text){
      SWG.alertErrorMessage("error.ip.invalid");
      return false;
    }
    return true;
  };

  this.validateHost = function(text) {
    if(PaidonUtil.trim(text) == ""){
      SWG.alertErrorMessage("error.host.empty");
      return false;
    }
    if(text.match(/^[a-zA-Z0-9!-~]+$/g) != text){
      SWG.alertErrorMessage("error.host.invalid");
      return false;
    }
    return true;
  };

  this.validateComment = function(text) {
    return true;
  };

  this.destroy = function(){
  };

};

function HostsFileTreeView(records, parent){
  this.records = records;
  this.parent = parent;
  this.logger = SWFactory.getLogger(this);
};

HostsFileTreeView.prototype = {

    get rowCount() { return this.records.length; },
    selection: null,
    getRowProperties: function(index, properties) {},
    getCellProperties: function(row, col, properties) {},
    getColumnProperties: function(col, properties) {},
    isContainer: function(index) { return false; },
    isContainerOpen: function(index) { return false; },
    isContainerEmpty: function(index) { return false; },
    isSeparator: function(index) {
        return this.records[index] == null;
    },
    isSorted: function() { return false; },
    canDrop: function(targetIndex, orientation) { return false; },
    drop: function(targetIndex, orientation) {},
    getParentIndex: function(rowIndex) { return -1; },
    hasNextSibling: function(rowIndex, afterIndex) { return false; },
    getLevel: function(index) { return 0; },
    getImageSrc: function(row, col) {},
    getProgressMode: function(row, col) {},
    getCellValue: function(row, col) { return this.getTextForCell(this.records[row], col.id); },
    getCellText: function(row, col) {
        return this.getTextForCell(this.records[row], col.id);
    },
    setTree: function(tree) {
        this._treeBoxObject = tree;
    },
    toggleOpenState: function(index) {},
    cycleHeader: function(col) {},
    selectionChanged: function() {},
    cycleCell: function(row, col) {},
    isEditable: function(row, col) { return true; },
    isSelectable: function(row, col) { return true; },
    setCellValue: function(row, col, value) {
        this.setTextForCell(this.records[row], col.id, value);
        this._treeBoxObject.invalidateCell(row, col);
    },
    setCellText: function(row, col, value) {
        this.setTextForCell(this.records[row], col.id, value);
        this._treeBoxObject.invalidateCell(row, col);
    },
    performAction: function(action) {},
    performActionOnRow: function(action, row) {},
    performActionOnCell: function(action, row, col) {},

  getTextForCell : function(record, col) {
    switch (col) {
      case "enable":return record.isEnable();
      case "ip":return record.getIp();
      case "host":return record.getHost();
      case "comment":return record.getComment();
    }
  },

  setTextForCell : function(record, col, value) {
    switch (col) {
      case "enable":
this.logger.debug("enable=" + value);
        return record.setEnable(value);
      case "ip":
        this.parent.validateIp(value);
        return record.setIp(value);
      case "host":
        this.parent.validateHost(value);
        return record.setHost(value);
      case "comment":
        this.parent.validateComment(value);
        return record.setComment(value);
    }
  },

  appendItem : function(record) {
    this.records.push(record);
    var newIdx = this.rowCount - 1;
    this._treeBoxObject.rowCountChanged(newIdx, 1);
    // select the new item now
    this.selection.select(newIdx);
    this._treeBoxObject.ensureRowIsVisible(newIdx);
    this._treeBoxObject.treeBody.focus();
  },

  removeItemAt : function(aRow) {
      this.records.splice(aRow, 1);
      this._treeBoxObject.rowCountChanged(aRow, -1);
      this.selection.clearSelection();
  },

  selectedIndexes : function() {
      var ret = [];
      var sel = this.selection;    // nsITreeSelection
      for (var rc = 0; rc < sel.getRangeCount(); rc++) {
          var start = {}, end = {};
          sel.getRangeAt(rc, start, end);
          for (var idx = start.value; idx <= end.value; idx++) {
              ret.push(idx);
          }
      }
      return ret;
  },

  getRecords : function() {
      return this.records;
  },

  destroy : function(){
  }

};

var main = new HostsFileEditDialog();
window.addEventListener("load", function(){
    main.init();
}, false);
window.addEventListener("unload", function(){
    main.destroy();
}, false);
