/**
  SwitchHosts
  Copyright (C) 2009-2009 Takashi Ishikawa.
  http://switchhosts.mozdev.org/

  This source code is released under the GPL license,
  available in the LICENSE file at the root of this installation
  and also online at http://www.gnu.org/licenses/gpl.txt
**/

Components.utils.import("resource://modules/PaidonUtil.jsm");
Components.utils.import("resource://modules/PaidonLogger.jsm");
Components.utils.import("resource://modules/SWConst.jsm");

var EXPORTED_SYMBOLS = ["PaidonDns"];

const Cc = Components.classes;
const Ci = Components.interfaces;
var Application = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);
var wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
var browser = wm.getEnumerator('navigator:browser').getNext().getBrowser();

function PaidonDns(listener) {

  //listeneronChangeLocation(sHost, aIp[])
  this.logger = new PaidonLogger(true);

  this.ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
  this.listener = listener;

    // shamelessly taken from flagfox extension
    this.LocationListener = {
        onLocationChange: function(aProgress, aRequest, aLocation){
            try {
                if (aLocation && aLocation.host && (aLocation.scheme != 'chrome') && (aLocation.scheme != 'file')) {
                    this.parent.locationChange(aLocation.host, SWConst.EVENT_LOCATION_CHANGE);
                }
                else {
                    this.parent.locationChange();
                }
            }
            catch (e) {
                this.parent.locationChange();
            }
        },
        onStateChange: function(a, b, c, d){
        },
        onProgressChange: function(a, b, c, d, e, f){
        },
        onStatusChange: function(a, b, c, d){
        },
        onSecurityChange: function(a, b, c){
        },
        onLinkIconAvailable: function(a){
        }

    };

    this.LocationListener.parent = this;

    this.notifyStateDocument = Ci.nsIWebProgress.NOTIFY_STATE_DOCUMENT;
    this.notifyLocation = Ci.nsIWebProgress.NOTIFY_LOCATION;

    browser.addProgressListener(this.LocationListener, this.notifyLocation | this.notifyStateDocument);

  this.refresh = function(){

        //Set offline
        this.ioService.offline = true;

        try{
          var cacheService = Cc["@mozilla.org/network/cache-service;1"].getService(Ci.nsICacheService);
          cacheService.evictEntries(Ci.nsICache.STORE_ANYWHERE);
        }finally{
          //Set online
          this.ioService.offline = false;
        }

        this.locationChange(this.getCurrentHost(), SWConst.EVENT_REFRESH_DNS);

        return true;
  };

    // update the statusbar panel
    this.locationChange = function(host, topic){
        if (!host) {
            return;
        }
        this.resolveIp(host, topic);
    };

    this.resolveIp = function(host, topic){
        try {
            //DNS Data Listener
            var dataListener = {
                data: [],
                QueryInterface: function(aIID){
                    if (aIID.equals(Components.interfaces.nsIDNSListener) ||
                        aIID.equals(Components.interfaces.nsISupports)) {
                        return this;
                    }
                    throw Components.results.NS_NOINTERFACE;
                },
                onLookupComplete: function(aRequest, aRecord, aStatus){
                    while (aRecord && aRecord.hasMore()) {
                        this.data.push(aRecord.getNextAddrAsString());
                    }
                    this.parent.listener.onChangeLocation(host, this.data, topic);
                    this.parent.currentHost = host;
                    this.parent.currentIps = this.data;
                }
            };
            dataListener.parent = this;
            dataListener.host = host;
            dataListener.topic = topic

            //Current Thread
            var target = Components.classes["@mozilla.org/thread-manager;1"].getService().currentThread;

            //DNS Service
            var dnsService = Components.classes["@mozilla.org/network/dns-service;1"].getService(Components.interfaces.nsIDNSService);
            try {
                dnsService.asyncResolve(host, 0, dataListener, target);
            }
            catch (e) {
                //Expected for unknown hosts
//                this.logger.debug("Async Resolve error: " + e)
            }
        }
        catch (e) {
            this.logger.error(e);
        }
    };

    this.getCurrentHost = function(){
      return this.currentHost;
    };

    this.getCurrentIps = function(){
      return this.currentIps;
    };

    this.destroy = function(){
        browser.removeProgressListener(this.Listener);
    };

};
