/**
  SwitchHosts
  Copyright (C) 2009-2009 Takashi Ishikawa.
  http://switchhosts.mozdev.org/

  This source code is released under the GPL license,
  available in the LICENSE file at the root of this installation
  and also online at http://www.gnu.org/licenses/gpl.txt
**/

Components.utils.import("resource://modules/PaidonUtil.jsm");
Components.utils.import("resource://modules/PaidonLogger.jsm");

var EXPORTED_SYMBOLS = ["PaidonFile"];

const Cc = Components.classes;
const Ci = Components.interfaces;
var Application = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);

function PaidonFile(strFilePath) {

  this.strFilePath = strFilePath;
  this.logger = new PaidonLogger(true);

  this.file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
  this.file.initWithPath( this.strFilePath );

  this.write = function(strOutputText, charset) {

    try{
      var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
                          .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
      if(!charset){
        charset = "UTF-8";
      }
      converter.charset = charset;

//      if (this.file.exists() == false) {
//        this.file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 664 );
//      }

      var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
      /* Open flags
      #define PR_RDONLY       0x01
      #define PR_WRONLY       0x02
      #define PR_RDWR         0x04
      #define PR_CREATE_FILE  0x08
      #define PR_APPEND      0x10
      #define PR_TRUNCATE     0x20
      #define PR_SYNC         0x40
      #define PR_EXCL         0x80
      */
      /*
      ** File modes ....
      **
      ** CAVEAT: 'mode' is currently only applicable on UNIX platforms.
      ** The 'mode' argument may be ignored by PR_Open on other platforms.
      **
      **   00400   Read by owner.
      **   00200   Write by owner.
      **   00100   Execute (search if a directory) by owner.
      **   00040   Read by group.
      **   00020   Write by group.
      **   00010   Execute by group.
      **   00004   Read by others.
      **   00002   Write by others
      **   00001   Execute by others.
      **
      */
      outputStream.init( this.file, 0x04 | 0x08 | 0x20, 00664, 0 );
      var chunk = converter.ConvertFromUnicode(strOutputText);
      var result = outputStream.write( chunk, chunk.length );
      outputStream.close();
      return result;
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }

  };

  this.createDirectory = function() {
    try{
      var LocalFileConstructor = new Components.Constructor(
                  "@mozilla.org/file/local;1", "nsILocalPath", "initWithFile");
      var localFile = new LocalFileConstructor(this.strFilePath);
      localFile.create(localFile.DIRECTORY_TYPE, 0777);
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.read = function(charset) {
    try{
      var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
                            .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
      if(!charset){
        charset = "UTF-8";
      }

      converter.charset = charset;

      if ( this.file.exists() == false ) {
        return null;
      }
      var is = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance( Components.interfaces.nsIFileInputStream );
      is.init( this.file, 0x01, 00004, null);
      var sis = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance( Components.interfaces.nsIScriptableInputStream );
      sis.init( is );
      var chunk = sis.read( sis.available() );
      var output = converter.ConvertToUnicode(chunk);
      sis.close();
      is.close();
      return output;
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.remove = function() {
    try{
      if ( this.file.exists() != false ) {
        this.file.remove(false);
      }
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.exists = function() {
    try{
      return this.file.exists();
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.getFiles = function(sExt) {
    try{
      if(!this.file.exists() || !this.file.isDirectory()){
        return new Array();
      }
      var list = this.file.directoryEntries;
      var files = new Array();
      while(list.hasMoreElements()){
        var childFile = list.getNext().QueryInterface(Components.interfaces.nsIFile);
        if(!childFile.isDirectory()){
          if(sExt != 'undefined'){
            if(PaidonUtil.endsWith(childFile.leafName, sExt)){
              files.push(childFile.leafName);
            }
          }else{
            files.push(childFile.leafName);
          }
        }
      }
      return files;
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.copy = function(destdir, destfilename)
  {
    try{
      if(!destfilename){
        destfilename = this.file.leafName;
      }

      // get a component for the directory to copy to
      var oDestFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
      oDestFile.initWithPath(destdir);
      oDestFile.append(destfilename);

      // get a component for the directory to copy to
      var oBackupFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
      oBackupFile.initWithPath(this.file.parent.path);
      oBackupFile.append(destfilename + ".oshosts_backup");

      if(oDestFile.exists()){
        //ꎞobNAbv쐬
        oDestFile.copyTo(oBackupFile.parent, destfilename + ".oshosts_backup");
      }

      try{
        //For windows
        if(oDestFile.exists()){
          try{ oDestFile.remove(false); }catch(e){};
        }
        this.file.copyTo(oDestFile.parent,destfilename);
      }catch(e){
        //sAꎞobNAbv猳ɖ߂
        if(oBackupFile.exists()){
          oBackupFile.copyTo(oDestFile.parent, destfilename);
        }
        throw e;
        // t@Cꍇ
        // e.result == Components.results.NS_ERROR_FILE_NOT_FOUND)
      }finally{
        if(oBackupFile.exists()){
          oBackupFile.remove(false);
        }
      }
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }

  };

  this.getLastModifiedTime = function()
  {
    try{
      return this.file.lastModifiedTime;
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.getFileSize = function()
  {
    try{
      return this.file.fileSize;
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.isReadable = function()
  {
    try{
      return this.file.isReadable();
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

  this.isWritable = function()
  {
    try{
      return this.file.isWritable();
    }catch(e){
      this.logger.error(e + "\nFilePath=" + this.strFilePath);
      throw e;
    }
  };

};
